#!/bin/bash
# -*- coding: utf-8 -*-

##
## @(#) リモートサーバ上でタスクスクリプトを実行する
##

set -e           # スクリプトにエラーがあれば終了
set -u           # 未定義の変数があればエラーにする
set -o pipefail  # パイプのコマンドが失敗したらエラー

## コマンドを表示してから実行
run () {
  echo -e "\033[34m[localhost]$ $1\033[m" >&2    # 青色で表示
  eval $@
}

## sshの設定
ssh_user="ubuntu"
ssh_host="localhost"
ssh_port=30022
ssh_key="/opt/var/vagrant/vagrant-ubuntu18.pem"
ssh_option=""

## sshオプションを追加
[[ -z "$ssh_key"  ]] || ssh_option="-i $ssh_key $ssh_option"
[[ -z "$ssh_port" ]] || ssh_option="-p $ssh_port $ssh_option"

## rsyncのオプション
##   -a: まるごとコピー、-v: 送信ファイル名を出力、-z: 圧縮して通信、
##   --delete: リストにないファイルは削除、-e: 接続に使うsshコマンド
rsync_option="-avz --delete -e 'ssh $ssh_option'"

## リモートサーバにコピーするファイル
files="tasks.txt tasks files lib/remote.sh"

## rsyncでリモートサーバにファイルをコピー
remote=${ssh_user}@${ssh_host}  # 例：ubuntu@localhost
run "rsync $rsync_option $files $remote:serversetup/"

## sshでリモートサーバに接続し、スクリプトを実行
cmd="sh remote.sh $remote"
run "ssh -t $ssh_option $remote 'cd serversetup; $cmd'"
