#!/bin/bash
# -*- coding: utf-8 -*-

##
## @(#) リモートサーバ上でタスクスクリプトを実行する
##

set -e           # スクリプトにエラーがあれば終了
set -u           # 未定義の変数があればエラーにする
set -o pipefail  # パイプのコマンドが失敗したらエラー

## エラーメッセージを表示して終了
abort () {
  echo "Error: $1" >&2
  exit 1
}

## コマンドを表示してから実行
run () {
  echo -e "\033[34m[localhost]$ $1\033[m" >&2    # 青色で表示
  eval $@
}

## sshの設定
sshconf="ssh.bash"
[[ -f $sshconf ]] || abort "$sshconf: file not found."
. $sshconf       # 設定ファイルを読み込む
: $ssh_user      # 変数が設定されていることを確認（されてないとエラー）
: $ssh_port
: $ssh_key
: $ssh_option

## sshオプションを追加
[[ -z "$ssh_key"  ]] || ssh_option="-i $ssh_key $ssh_option"
[[ -z "$ssh_port" ]] || ssh_option="-p $ssh_port $ssh_option"

## rsyncのオプション
##   -a: まるごとコピー、-v: 送信ファイル名を出力、-z: 圧縮して通信、
##   --delete: リストにないファイルは削除、-e: 接続に使うsshコマンド
rsync_option="-avz --delete -e 'ssh $ssh_option'"

## リモートサーバにコピーするファイル
files="tasks.txt tasks files lib/remote.sh"

## サーバ名をホストファイルから読み出す
## （注：while文だとsshの -t オプションがエラーになるので、whileを避ける）
hostsfile="hosts.txt"
[[ -f $hostsfile ]] || abort "$hostsfile: file not found."
for ssh_host in `grep -Ev '^#|^[ \t]*$' $hostsfile`; do

  ## rsyncでリモートサーバにファイルをコピー
  remote=${ssh_user}@${ssh_host}  # 例：ubuntu@localhost
  run "rsync $rsync_option $files $remote:serversetup/"

  ## sshでリモートサーバに接続し、スクリプトを実行
  cmd="sh remote.sh $remote"
  run "ssh -t $ssh_option $remote 'cd serversetup; $cmd'"

done
