#!/bin/sh
# -*- coding: utf-8 -*-

## @(#) リモートサーバ上でタスクスクリプトを起動する

host=$1          # 例：ubuntu@localhost

set -eu

## 色つきで表示
colorize () {
  echo "\033[35m$1\033[m"     # マジェンタ（赤紫）
}

## ログディレクトリがなければ作成
logdir=/var/log/serversetup
[ -d $logdir ] || { sudo mkdir $logdir; sudo chown $USER $logdir; }

## 名前つきパイプのファイル名
pipe="_pipe.$$"                   # プロセスID ($$) を使って重複を避ける
trap "rm -f $pipe" 0              # このスクリプト終了時に確実に削除する

## タスクファイルに書かれた順番に、タスクスクリプトを実行する
grep -Ev '^#|^[ \t]*$' tasks.txt | while read script; do
  timestamp=`LC_ALL=C date +'%Y%m%d%H%M%S'`
  log=$logdir/${timestamp}_${script%.*}.log
  #bash tasks/$script | tee $log  # パイプはスクリプトの実行失敗を検出できず。
  mkfifo $pipe                    # かわりに名前付きパイプを作成し、
  tee -a $log < $pipe &           # リダイレクトでパイプをシミュレート。
  case $script in
  *.bash|*.sh|*.tcsh)             # 拡張子をそのままコマンド名にできる場合、
    cmd=${script##*.}             # 変数展開で拡張子を取り出し、コマンドとする
    colorize "[$host] $cmd -e tasks/$script | tee $log" >&2
    $cmd -e tasks/$script > $pipe 2>&1   # スクリプトの実行失敗を検出！
    ;;
  *)                              # 拡張子からはコマンド名がわからない場合、
    chmod +x tasks/$script        # スクリプトに実行属性をつけて実行
    colorize "[$host] tasks/$script | tee $log" >&2
    tasks/$script > $pipe 2>&1           # スクリプトの実行失敗を検出！
    ;;
  esac
  rm $pipe
done
