# -*- coding: utf-8 -*-
## シェルオプション
set -e           # コマンド実行に失敗したらエラー
set -u           # 未定義の変数にアクセスしたらエラー
set -o pipefail  # パイプのコマンドが失敗したらエラー（bashのみ）

## エラーメッセージを表示して終了
abort () {
  echo $1 2>&1
  exit 1
}

## 引数で指定されたコマンドを表示して実行するシェル関数
## （ここで「$PS4」には「+」が入っているのでそれを使う）
+ () {   # 関数名が「+」であるような関数を定義（bash only）
  if [[ $# -eq 1 ]]; then      # 引数が1個のとき
    echo $PS4 "$1"; eval "$1"
  else                         # 引数が2個以上のとき
    echo $PS4 $@  ; eval $@
  fi
}

## 引数を色つきで表示する（注：bashでのみ動作）
:: () {
  echo -e "\033[33m::" $@ "\033[m"   # yellow
  #echo -e "\033[37m::" $@ "\033[m"   # white
}

## 指定時間以内に apt-get update を実行していれば終了ステータスが 0
apt_updated_recently () {
  ## ディレクトリが指定時間内に更新されていれば正常終了
  local minute=${1:-$APT_UPDATED_MINUTES}  # 第1引数は分（デフォルト60分）
  local dir="/var/lib/apt/lists/partial"
  local out=`find $dir -maxdepth 0 -mmin "-$minute"`
  [[ -n "$out" ]] && return 0  # 正常終了
  return 1                     # エラー終了
}

APT_UPDATED_MINUTES=60    # デフォルト60分

## 指定したパッケージがすべてインストール済みなら正常終了、
## インストールされてないパッケージが1つでもあればエラー終了
## （もしすべてインストール済みなら apt-get install をスキップできる）
apt_installed () {
  local pkg
  for pkg in $@; do
    dpkg -l | grep -q "^ii  $pkg " || return 1   # エラー終了
  done
  return 0  # 正常終了
}

## 再起動を表す終了ステータス
REBOOT_EXIT=246
