# -*- coding: utf-8 -*-
. `dirname $0`/../lib/common.bash   # 共通の関数や定数
. `dirname $0`/../vars/vars.bash    # 環境別の変数
: $sshd_port     # 変数が設定されていることを確認（されてないとエラー）

## sshdでオフにすべきオプション
## （rootユーザによるログイン、チャレンジレスポンス認証、
##   パスワード認証）
options="PermitRootLogin
         ChallengeResponseAuthentication
         PasswordAuthentication"

## 設定ファイル名
conf="sshd_config"

:: パッチファイルをコピー（あれば）; (
  if [[ -f files/sshd/sshd_config.patch ]]; then
    + "sudo cp {files/sshd,/etc/ssh}/$conf.patch"
  fi
)

:: ディレクトリを移動; {
  + cd /etc/ssh/
}

:: オリジナルのsshd設定ファイルをバックアップ; (
  if [[ ! -f $conf.original ]]; then
    + sudo mv $conf $conf.original # これは最初の1回だけ
  fi
)

:: オリジナルをコピーして新しいsshd設定ファイルを作成; (
  + sudo cp $conf.original $conf   # これは毎回行う
)

## パッチファイルがあれば
if [[ -f $conf.patch ]]; then

  :: 差分パッチをsshd設定ファイルに適用; (
    + "sudo patch < $conf.patch"
  )

## パッチファイルがなければ
else

  :: sshd設定ファイルを直接変更; (
    ## rootユーザによるログイン、チャレンジレスポンス認証、
    ## パスワード認証をオフにする
    for x in $options; do
      + "sudo sed -i -e 's/^#*${x} .*/${x} no/' $conf"
    done
    ## ポート番号を変更
    + "sudo sed -i -e 's/^#*Port .*/Port ${sshd_port}/' $conf"
  )

fi

:: 設定ファイルを正しく変更できたことを確認; (
  for x in $options; do
    newval=`awk "/^${x} /"'{print $2}' $conf`
    + "[ 'no' = '$newval' ]  # new value should be 'no'"
  done
  + "[ '${sshd_port}'_ = `awk '/^Port /{print \$2}' $conf`_ ]"
)

+ cd -

:: sshdデーモンプロセスを再起動し設定を反映; (
  + sudo service ssh restart   # 毎回行う（変更がなくても気にしない）
)
